 /*******************************************************************************
* FILE NAME: sensor_routines.c
*
* DESCRIPTION:
*  This file contains sensor related routines.
*
* USAGE:
*
*******************************************************************************/

#include <stdio.h>
#include "ifi_utilities.h"
#include "ifi_aliases.h"
#include "ifi_default.h"

#include "user_routines.h"
#include "clock_routines.h"

#define VARIABLE_DECLARE  /* causes certain global variables to be declared only once */
  #include "sensor_routines.h"
#undef VARIABLE_DECLARE

#define END_STARTUP 80   // Sample gyro over n cycles to obtain accurate GyroNeutral
#define SETTLE_STARTUP 40  // Allow the gyro to settle down for n cycles after power-up
                           // An ADXRS150 stablizes by .1 seconds
#define SAMPLE_RATE 40/250   // Sample rate of the gyro - gyro provides samples at 40Hz, we check at 250Hz

/*******************************************************************************
* FUNCTION NAME: Gyro
* PURPOSE:       Initializes and accumulates the gyro
* CALLED FROM:   user_routines_fast.c
* NOTES:
*        The gyro has a range of 1007 to 3 and a neutral point of 519-521
*        
*******************************************************************************/
void Gyro()
{
  static long LastGyro;
  long CurrentGyro;
  static unsigned long LastClocksec=0, LastClockms=0;
  static unsigned long GyroStartup=0;

  if (GyroTicks > 0)
  {
    /** Update gyro **/
    INTCONbits.GIEL = 0;	/* Disable Low Priority Interrupts */
    GyroTicks--;		/* Avoid the problem of decrementing it during an interrupt */
    INTCONbits.GIEL = 1;	/* Enable Low Priority Interrupts */

//    if (GyroTicks > 2)
//    {
//      PrintString((char *) " ***WARNING*** Gyro falling behind ");
//      PrintDecimal(GyroTicks);
//      PrintCR();
//    }

    if (GyroStartup == END_STARTUP)
    {
      CurrentGyro = (long)Get_Analog_Value(rc_ana_in01);
      if (min_gyro > CurrentGyro) min_gyro = CurrentGyro;
      if (max_gyro < CurrentGyro) max_gyro = CurrentGyro;
      CurrentGyro -= GyroNeutral;

      CurrentGyro = CurrentGyro * SAMPLE_RATE; // Add decimal points of accuracy
      GyroSum += (CurrentGyro + LastGyro) / 2;  // Is this helping or hurting?
//      GyroSum += CurrentGyro;
      LastGyro = CurrentGyro;

      if (LastClocksec < Clocksec)
      {
        LastClocksec = Clocksec;
        printf("GyroSum= %ld, degrees=%d\n\r",GyroSum,Gyro_Degrees());
      }
    }
    else /** Initializing Gyro **/
    {
      GyroStartup++;
      /** Sample over n cycles to obtain accurate GyroNeutral **/
      /** Allow n cycles for the gyro to be properly powered-up and settle down **/
      if (GyroStartup > SETTLE_STARTUP)
      {
        GyroNeutral += Get_Analog_Value(rc_ana_in01)<<6; // Increase accuracy
        if (GyroStartup == END_STARTUP)
        {
          GyroNeutral /= (END_STARTUP-SETTLE_STARTUP);
          GyroNeutral = (GyroNeutral+32)>>6; // Round to original accuracy
          min_gyro = 1024;
          max_gyro = 0;
          LastGyro = 0;
          GyroSum = 0;
        }
      }
    }
  }
}

/*******************************************************************************
* FUNCTION NAME: Gyro_Degrees
* PURPOSE:       Convert gyrosum to degrees in the range -180 to 180
* NOTES:         By definition 0 degrees is the original robot heading
*          
*******************************************************************************/
//  ************** ADXRS150
//  #define DEG90   8700  // Gyro ticks in  90 degrees
//  #define DEG180 17400  // Gyro ticks in 180 degrees
//  #define DEG360 34800  // Gyro ticks in 360 degrees
//  ************** BEI Gyrochip 2003
//  #define DEG90  20852  // Gyro ticks in  90 degrees
//  #define DEG180 41704  // Gyro ticks in 180 degrees
//  #define DEG360 83407  // Gyro ticks in 360 degrees
//  ************** ADXRS300
  #define DEG90  3300  // Gyro ticks in  90 degrees
  #define DEG180 6600  // Gyro ticks in 180 degrees
  #define DEG360 13200  // Gyro ticks in 360 degrees

int Gyro_Degrees()
{
  long temp;

  temp = GyroSum % DEG360;

  if (temp > DEG180)
    temp = (DEG360 - temp) * -1;
  else if (temp < -DEG180)
    temp = DEG360 + temp;
  else if (temp == -DEG180)
    temp = DEG180;

  //Reverse reading if the gyro is mounted upside down
  temp *= -1;

  return temp * 180 / DEG180;
}


/*******************************************************************************
*	FUNCTION:		Initialize_Interrupts()
*	PURPOSE:		Initializes the interrupt hardware.
*	CALLED FROM:	user_routines.c/User_Initialization()
*******************************************************************************/
void Initialize_Interrupts(void)  
{
	// initialize external interrupt 1 (INT2 on user 18F8520)
	TRISBbits.TRISB2 = 1;		// make sure the RB2/INT2 pin is configured as an input [108]
								//
	INTCON3bits.INT2IP = 0;		// 0: interrupt 1 is low priority (leave at 0 for IFI controllers) [91]
								// 1: interrupt 1 is high priority
								//
	INTCON2bits.INTEDG2 = 1;	// 0: trigger on the falling-edge [90]
								// 1: trigger on the rising-edge
								//
	INTCON3bits.INT2IE = 1;		// 0: disable interrupt	1 [91]
								// 1: enable interrupt 1
	
	// initialize external interrupt 2 (INT3 on user 18F8520)
	TRISBbits.TRISB3 = 1;		// make sure the RB3/CCP2/INT3 pin is configured as an input [108]
								//
	INTCON2bits.INT3IP = 0;		// 0: interrupt 2 is low priority (leave at 0 for IFI controllers) [90]
								// 1: interrupt 2 is high priority
								//
	INTCON2bits.INTEDG3 = 1;	// 0: trigger on the falling-edge [90]
								// 1: trigger on the rising-edge
								//
	INTCON3bits.INT3IE = 1;		// 0: disable interrupt	2 [91]
								// 1: enable interrupt 2
}
